package net.hostsharing.hsadminng.hs.office.partner;

import io.micrometer.core.annotation.Timed;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.Repository;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

public interface HsOfficePartnerRepository extends Repository<HsOfficePartnerEntity, UUID> {

    @Timed("app.office.partners.repo.findByUuid")
    Optional<HsOfficePartnerEntity> findByUuid(UUID id);

    @Timed("app.office.partners.repo.findAll")
    List<HsOfficePartnerEntity> findAll(); // TODO.refa: move to a repo in test sources

//    @Query("""
//            SELECT partner FROM HsOfficePartnerEntity partner
//                JOIN HsOfficeRelationRealEntity rel ON rel.uuid = partner.partnerRel.uuid
//                JOIN HsOfficeContactRealEntity contact ON contact.uuid = rel.contact.uuid
//                JOIN HsOfficePersonRealEntity person ON person.uuid = rel.holder.uuid
//                LEFT JOIN HsOfficePartnerDetailsEntity details ON partner.details.uuid = details.uuid
//                WHERE :name is null
//                    OR (details IS NOT NULL AND details.birthName like concat(cast(:name as text), '%'))
//                    OR contact.caption like concat(cast(:name as text), '%')
//                    OR person.tradeName like concat(cast(:name as text), '%')
//                    OR person.givenName like concat(cast(:name as text), '%')
//                    OR person.familyName like concat(cast(:name as text), '%')
//            """)
    @Query(value = """
            select partner.uuid, partner.detailsuuid, partner.partnernumber, partner.partnerreluuid, partner.version
              from hs_office.partner_rv partner
                       join hs_office.relation partnerRel on partnerRel.uuid = partner.partnerreluuid
                       join hs_office.contact contact on contact.uuid = partnerRel.contactuuid
                       join hs_office.person partnerPerson on partnerPerson.uuid = partnerRel.holderuuid
                       left join hs_office.partner_details_rv partnerDetails on partnerDetails.uuid = partner.detailsuuid
              where :name is null
                  or (partnerDetails.uuid is not null and partnerDetails.birthname like (cast(:name as text) || '%') escape '')
                  or contact.caption like (cast(:name as text) || '%') escape ''
                  or partnerPerson.tradename like (cast(:name as text) || '%') escape ''
                  or partnerPerson.givenname like (cast(:name as text) || '%') escape ''
                  or partnerPerson.familyname like (cast(:name as text) || '%') escape ''
            """, nativeQuery = true)
    @Timed("app.office.partners.repo.findPartnerByOptionalNameLike")
    List<HsOfficePartnerEntity> findPartnerByOptionalNameLike(String name);

    @Timed("app.office.partners.repo.findPartnerByPartnerNumber")
    Optional<HsOfficePartnerEntity> findPartnerByPartnerNumber(Integer partnerNumber);

    @Timed("app.office.partners.repo.save")
    HsOfficePartnerEntity save(final HsOfficePartnerEntity entity);

    @Timed("app.office.partners.repo.count")
    long count();

    @Timed("app.office.partners.repo.deleteByUuid")
    int deleteByUuid(UUID uuid);
}
